﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Models.Rules
{
    public class Rules
    {
        public static bool NearCell(Cell playerChoice, List<Cell> cells)
        {
            return cells.Any(item => playerChoice.Pos.X == item.Pos.X + 1 || playerChoice.Pos.X == item.Pos.X - 1 || playerChoice.Pos.Y == item.Pos.Y + 1 || playerChoice.Pos.Y == item.Pos.Y - 1);
        }

        public static bool IsCellEmpty(Cell playerChoice)
        {
            return playerChoice.Value == null;
        }

        public static bool IsCellValid(Cell playerChoice, List<Cell> cells)
        {
            return NearCell(playerChoice, cells) && IsCellEmpty(playerChoice);
        }
        
        public static bool IsZone(Cell playerChoice, List<Cell> cells)
        {
            foreach (var item in cells)
            {
                if (playerChoice.Pos.X != item.Pos.X + 1 && playerChoice.Pos.X != item.Pos.X - 1
                                                         && playerChoice.Pos.Y != item.Pos.Y + 1 &&
                                                         playerChoice.Pos.Y != item.Pos.Y - 1) continue;
                if (playerChoice.Value == item.Value)
                {
                    return true;
                }
            }
            return false;
        }
    
        public static bool IsRopePath(Cell playerChoice, List<Cell> cells, List<Cell> ropePaths)
        {
            foreach (var item in cells)
            {
                if (playerChoice.Pos.X != item.Pos.X + 1 && playerChoice.Pos.X != item.Pos.X - 1
                                                         && playerChoice.Pos.Y != item.Pos.Y + 1 &&
                                                         playerChoice.Pos.Y != item.Pos.Y - 1) continue;
                
                if (playerChoice.Value != item.Value + 1 && playerChoice.Value != item.Value - 1) continue;
                
                foreach (var path in ropePaths)
                {
                    if (path.Equals(playerChoice))
                    {
                        return true;
                    }
                }
                return IsRopePath(item, cells, ropePaths);
            }
            return false;
        }
        
        public static int HowMany(Cell playerChoice, List<Cell> cells) 
        { 
            foreach(var pos in cells)
            {
                if (pos.Equals(playerChoice))
                {
                    return pos.GetCellType() ? 6 : 12;
                }
            }
            return 0;
        }

        public static void SetValueAndPenalty(int valueChoice, Cell playerChoice, List<Cell> cells)
        {
            int val = HowMany(playerChoice, cells);
            
            foreach (var pos in cells)
            {
                if (!pos.Equals(playerChoice)) continue;
                
                if (valueChoice > val)
                {
                    playerChoice.Background = "penalty";
                }
                playerChoice.Value = valueChoice;
            }
        }
    }
}
